# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pilotbuild.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='pilotbuild.proto',
  package='',
  syntax='proto3',
  serialized_options=b'\252\002\016GrpcPilotBuild',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10pilotbuild.proto\"?\n\x06Module\x12\x0e\n\x06number\x18\x01 \x01(\x05\x12\x0b\n\x03\x66id\x18\x02 \x01(\t\x12\x0b\n\x03hid\x18\x03 \x01(\t\x12\x0b\n\x03uid\x18\x04 \x01(\t\"|\n\x0c\x42uildRequest\x12\x1a\n\x06target\x18\x01 \x01(\x0e\x32\n.TargetMcu\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x18\n\x07modules\x18\x03 \x03(\x0b\x32\x07.Module\x12\x0e\n\x06source\x18\x04 \x01(\x0c\x12\x15\n\rmainboard_uid\x18\x05 \x01(\t\"Z\n\x0b\x42uildResult\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x1c\n\x07\x62intype\x18\x02 \x01(\x0e\x32\x0b.BinaryType\x12\x0c\n\x04hash\x18\x03 \x01(\t\x12\x0e\n\x06\x62inary\x18\x04 \x01(\x0c\"S\n\x0b\x42uildStatus\x12\x1a\n\x04step\x18\x01 \x01(\x0e\x32\n.BuildStepH\x00\x12\x1e\n\x06result\x18\x02 \x01(\x0b\x32\x0c.BuildResultH\x00\x42\x08\n\x06status*+\n\tTargetMcu\x12\x0f\n\x0bSTM32F103RC\x10\x00\x12\r\n\tSTM32F40X\x10\x01*\xd2\x02\n\tBuildStep\x12\x08\n\x04\x44one\x10\x00\x12\r\n\tQueueFull\x10\x01\x12\t\n\x05\x45rror\x10\x02\x12\n\n\x06Queued\x10\x03\x12\x13\n\x0fValidateRequest\x10\x04\x12\x1c\n\x18\x43leanCpldOutputDirectory\x10\x05\x12\x16\n\x12GenerateCpldSource\x10\x06\x12\x15\n\x11\x43ompileCpldSource\x10\x07\x12\x17\n\x13PackageCpldFirmware\x10\x08\x12\x15\n\x11PackageCpldSource\x10\t\x12\x1b\n\x17\x43leanStmOutputDirectory\x10\n\x12\x15\n\x11GenerateStmSource\x10\x0b\x12\x14\n\x10\x43ompileStmSource\x10\x0c\x12\x16\n\x12PackageStmFirmware\x10\r\x12\x14\n\x10PackageStmSource\x10\x0e\x12\x0b\n\x07\x43leanup\x10\x0f*h\n\nBinaryType\x12\x0f\n\x0bMCUFirmware\x10\x00\x12\r\n\tMCUSource\x10\x01\x12\x11\n\rFPGABitstream\x10\x02\x12\x0e\n\nFPGASource\x10\x03\x12\r\n\tVariables\x10\x04\x12\x08\n\x04\x44ocs\x10\x05\x32\x34\n\nPilotBuild\x12&\n\x05\x42uild\x12\r.BuildRequest\x1a\x0c.BuildStatus0\x01\x42\x11\xaa\x02\x0eGrpcPilotBuildb\x06proto3'
)

_TARGETMCU = _descriptor.EnumDescriptor(
  name='TargetMcu',
  full_name='TargetMcu',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STM32F103RC', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STM32F40X', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=388,
  serialized_end=431,
)
_sym_db.RegisterEnumDescriptor(_TARGETMCU)

TargetMcu = enum_type_wrapper.EnumTypeWrapper(_TARGETMCU)
_BUILDSTEP = _descriptor.EnumDescriptor(
  name='BuildStep',
  full_name='BuildStep',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Done', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='QueueFull', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Error', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Queued', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ValidateRequest', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CleanCpldOutputDirectory', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GenerateCpldSource', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CompileCpldSource', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PackageCpldFirmware', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PackageCpldSource', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CleanStmOutputDirectory', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GenerateStmSource', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CompileStmSource', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PackageStmFirmware', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PackageStmSource', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Cleanup', index=15, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=434,
  serialized_end=772,
)
_sym_db.RegisterEnumDescriptor(_BUILDSTEP)

BuildStep = enum_type_wrapper.EnumTypeWrapper(_BUILDSTEP)
_BINARYTYPE = _descriptor.EnumDescriptor(
  name='BinaryType',
  full_name='BinaryType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='MCUFirmware', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MCUSource', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FPGABitstream', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FPGASource', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Variables', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Docs', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=774,
  serialized_end=878,
)
_sym_db.RegisterEnumDescriptor(_BINARYTYPE)

BinaryType = enum_type_wrapper.EnumTypeWrapper(_BINARYTYPE)
STM32F103RC = 0
STM32F40X = 1
Done = 0
QueueFull = 1
Error = 2
Queued = 3
ValidateRequest = 4
CleanCpldOutputDirectory = 5
GenerateCpldSource = 6
CompileCpldSource = 7
PackageCpldFirmware = 8
PackageCpldSource = 9
CleanStmOutputDirectory = 10
GenerateStmSource = 11
CompileStmSource = 12
PackageStmFirmware = 13
PackageStmSource = 14
Cleanup = 15
MCUFirmware = 0
MCUSource = 1
FPGABitstream = 2
FPGASource = 3
Variables = 4
Docs = 5



_MODULE = _descriptor.Descriptor(
  name='Module',
  full_name='Module',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='number', full_name='Module.number', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fid', full_name='Module.fid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hid', full_name='Module.hid', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uid', full_name='Module.uid', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20,
  serialized_end=83,
)


_BUILDREQUEST = _descriptor.Descriptor(
  name='BuildRequest',
  full_name='BuildRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='target', full_name='BuildRequest.target', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='version', full_name='BuildRequest.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='modules', full_name='BuildRequest.modules', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source', full_name='BuildRequest.source', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mainboard_uid', full_name='BuildRequest.mainboard_uid', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=85,
  serialized_end=209,
)


_BUILDRESULT = _descriptor.Descriptor(
  name='BuildResult',
  full_name='BuildResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='BuildResult.version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bintype', full_name='BuildResult.bintype', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hash', full_name='BuildResult.hash', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='binary', full_name='BuildResult.binary', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=211,
  serialized_end=301,
)


_BUILDSTATUS = _descriptor.Descriptor(
  name='BuildStatus',
  full_name='BuildStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='step', full_name='BuildStatus.step', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='result', full_name='BuildStatus.result', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='status', full_name='BuildStatus.status',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=303,
  serialized_end=386,
)

_BUILDREQUEST.fields_by_name['target'].enum_type = _TARGETMCU
_BUILDREQUEST.fields_by_name['modules'].message_type = _MODULE
_BUILDRESULT.fields_by_name['bintype'].enum_type = _BINARYTYPE
_BUILDSTATUS.fields_by_name['step'].enum_type = _BUILDSTEP
_BUILDSTATUS.fields_by_name['result'].message_type = _BUILDRESULT
_BUILDSTATUS.oneofs_by_name['status'].fields.append(
  _BUILDSTATUS.fields_by_name['step'])
_BUILDSTATUS.fields_by_name['step'].containing_oneof = _BUILDSTATUS.oneofs_by_name['status']
_BUILDSTATUS.oneofs_by_name['status'].fields.append(
  _BUILDSTATUS.fields_by_name['result'])
_BUILDSTATUS.fields_by_name['result'].containing_oneof = _BUILDSTATUS.oneofs_by_name['status']
DESCRIPTOR.message_types_by_name['Module'] = _MODULE
DESCRIPTOR.message_types_by_name['BuildRequest'] = _BUILDREQUEST
DESCRIPTOR.message_types_by_name['BuildResult'] = _BUILDRESULT
DESCRIPTOR.message_types_by_name['BuildStatus'] = _BUILDSTATUS
DESCRIPTOR.enum_types_by_name['TargetMcu'] = _TARGETMCU
DESCRIPTOR.enum_types_by_name['BuildStep'] = _BUILDSTEP
DESCRIPTOR.enum_types_by_name['BinaryType'] = _BINARYTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Module = _reflection.GeneratedProtocolMessageType('Module', (_message.Message,), {
  'DESCRIPTOR' : _MODULE,
  '__module__' : 'pilotbuild_pb2'
  # @@protoc_insertion_point(class_scope:Module)
  })
_sym_db.RegisterMessage(Module)

BuildRequest = _reflection.GeneratedProtocolMessageType('BuildRequest', (_message.Message,), {
  'DESCRIPTOR' : _BUILDREQUEST,
  '__module__' : 'pilotbuild_pb2'
  # @@protoc_insertion_point(class_scope:BuildRequest)
  })
_sym_db.RegisterMessage(BuildRequest)

BuildResult = _reflection.GeneratedProtocolMessageType('BuildResult', (_message.Message,), {
  'DESCRIPTOR' : _BUILDRESULT,
  '__module__' : 'pilotbuild_pb2'
  # @@protoc_insertion_point(class_scope:BuildResult)
  })
_sym_db.RegisterMessage(BuildResult)

BuildStatus = _reflection.GeneratedProtocolMessageType('BuildStatus', (_message.Message,), {
  'DESCRIPTOR' : _BUILDSTATUS,
  '__module__' : 'pilotbuild_pb2'
  # @@protoc_insertion_point(class_scope:BuildStatus)
  })
_sym_db.RegisterMessage(BuildStatus)


DESCRIPTOR._options = None

_PILOTBUILD = _descriptor.ServiceDescriptor(
  name='PilotBuild',
  full_name='PilotBuild',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=880,
  serialized_end=932,
  methods=[
  _descriptor.MethodDescriptor(
    name='Build',
    full_name='PilotBuild.Build',
    index=0,
    containing_service=None,
    input_type=_BUILDREQUEST,
    output_type=_BUILDSTATUS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_PILOTBUILD)

DESCRIPTOR.services_by_name['PilotBuild'] = _PILOTBUILD

# @@protoc_insertion_point(module_scope)



def getDevice(model, module, compiler, helpers):
  return O8Device(model, module, compiler, helpers)

def toGPIO(this, items):
  return 'GPIO' + chr(items+65)

def default_config():
  return {}

class O8Device():
  size = 1
  ctype = 'uint8_t'
  rusttype = 'u8'
  length = 1
  include = ['stm32f10x.h'] 
  init_source = ''
  dev_to_mem_source = ''
  mem_to_dev_source = ''
  mem_doc = { "read": [], "write": [] }

  decl = {
    'c': { 'name': 'o8_t', 'decl': 'typedef uint8_t o8_t;' },
    'rust': { 'name': 'u8', 'decl': '' }
  }

  module = None
  model = None
  compiler = None
  helpers = None 
  
  def __init__(self, model, module, compiler, helpers):
    self.size = 1
    self.module = module
    self.helpers = {'gpio': toGPIO, **helpers}
    self.compiler = compiler
    self.model = model

  def compile(self):
    inputs_template = self.compiler.compile("""// input source for device {{device.name}}
  {{#each device.hw.Inputs}}
  BITBAND_SRAM(&plc_mem_devices.m{{../device.slot}}, {{IO}}) = BITBAND_PERI({{gpio GPIO}}_BASE + 8, {{Pin}});
{{/each}}
    """)

    outputs_template = self.compiler.compile("""// output source for device {{device.name}}
  {{#each device.hw.Outputs}}
  GPIO_OUT_SET({{gpio GPIO}}_BASE, {{Pin}}, BITBAND_SRAM(&plc_mem_devices.m{{../device.slot}}, {{IO}}));
{{/each}}
    """)
    self.dev_to_mem_source = inputs_template(self.module, self.helpers)
    self.mem_to_dev_source = outputs_template(self.module, self.helpers)
    self.mem_doc['read'] = [{ "name": "i{}".format(i), "desc": "digital input {}".format(i), "byte": 0, "bit": i, "length": 1, "signed": False, "datatype": "bool"} for i in range(4)]
    self.mem_doc['write'] = ([{ "name": "o{}".format(i), "desc": "digital output {}".format(i), "byte": 0, "bit": i, "length": 1, "signed": False, "datatype": "bool"} for i in range(4)])


import os
from .pilotdriver import PilotDriver

def setup_arguments(parser):
  parser.add_argument('--fw-version', '-f', dest='fwversion',
                  help='Specify firmware version, latest is used if not specified')
  parser.add_argument('--workdir', '-d', default=os.getcwd(), dest='workdir',
                  help='Set working directory')
  parser.add_argument('--wait_bootmsg', '-w', dest='wait_bootmsg', action='store_true',
                         help='Wait for Pilot boot message and display after reboot')
  parser.add_argument('--source', '-c', default=None, dest='source',
                      help='Download Sourcecode only')
  parser.add_argument('--regnode', '-n', dest='regnode', action='store_true',
                      help='Register node')
  #parser.add_argument('--reset', '-r', default=None, action='store_const', const='reset', dest='reset',
  #                    help='Resets the Pilot Mainboard')
  parser.add_argument('--yes', '-y', default=None, action='store_const', const='noninteractive', dest='noninteractive',
                      help='Confirms default action (non-interactive mode)')
  parser.add_argument('--driveronly', '-x', default=None, action='store_const', const='driveronly', dest='driveronly',
                      help='Installs driver only')
  for mod in range(1, PilotDriver.MODULE_COUNT+1):
    parser.add_argument('--module{}'.format(mod), '-m{}'.format(mod), dest='m{}'.format(mod), help='Firmware id of module {}'.format(mod))
    
def reset_arguments(parser):
  parser.add_argument('--wait_bootmsg', '-w', dest='wait_bootmsg', action='store_true',
                         help='Wait for Pilot boot message and display after reboot')

def module_arguments(parser):
  parser.add_argument('module', metavar='N', type=int,
                    help='module number')
  parser.add_argument('--pinout', help='Show Module Pinout', action='store_true')
  parser.add_argument('--usage', help='Show Module Usage Examples', action='store_true')

def program_arguments(parser):
  parser.add_argument('--fw-version', '-f', dest='fwversion',
                  help='Specify firmware version, latest is used if not specified')
  parser.add_argument('--workdir', '-d', default=os.getcwd(), dest='workdir',
                  help='Set working directory')
  parser.add_argument('--wait_bootmsg', '-w', dest='wait_bootmsg', action='store_true',
                         help='Wait for Pilot boot message and display after reboot')
  parser.add_argument('name', metavar='node', type=str, nargs='?',
                      help='Specify named node to program')
  parser.add_argument('--binary', '-b', default=None, dest='bin',
                      help='Write binary image to the Pilot Microcontroller')
  parser.add_argument('--logicbinary', '-l', default=None, dest='logicbin',
                      help='Write bitstream to the Pilot CPLD')
  parser.add_argument('--variables', '-v', default=None, dest='vars',
                      help='Set PLC variables')
  parser.add_argument('--doc', default=None, dest='doc',
                      help='set firmware config file')
  parser.add_argument('--config', dest='configfile',
                    default=None, help='module config file (.pilotfwconfig.json)')
  parser.add_argument('--mcuonly', action='store_const', default=False,
                      const=True, help='Program MCU only')

def compiler_arguments(parser):
  parser.add_argument('--fw-version', '-f', dest='fwversion',
                  help='Specify firmware version, latest is used if not specified')
  parser.add_argument('--workdir', '-d', default=os.getcwd(), dest='workdir',
                  help='Set working directory')
  parser.add_argument('--wait_bootmsg', '-w', dest='wait_bootmsg', action='store_true',
                         help='Wait for Pilot boot message and display after reboot')
  parser.add_argument('--config', dest='configfile',
                      default=None, help='module config file (.pilotfwconfig.json)')
  parser.add_argument('--iec2c', dest='iec2cdir',
                      default=None, help='directory of iec2c compiler')
  parser.add_argument('--source', dest='source',
                      default=None, help='source directory')
  parser.add_argument('--target', dest='target',
                      default=None, help='target directory')
  parser.add_argument('--verbose', action='store_const',
                      const=True, help='verbose output')
  parser.add_argument('files', nargs='*', default=None,
                      help='IEC Structured Text File')

def project_arguments(parser):
  parser.add_argument('--fw-version', '-f', dest='fwversion',
                  help='Set working directory')
  parser.add_argument('--workdir', '-d', default=os.getcwd(), dest='workdir',
                  help='Set working directory')
  parser.add_argument('--wait_bootmsg', '-w', dest='wait_bootmsg', action='store_true',
                         help='Wait for Pilot boot message and display after reboot')
  parser.add_argument('--compiler', dest='compiler', help='Specify compiler to use. Run --show-compilers to get a list of options')
  parser.add_argument('--config', dest='configfile',
                      default=None, help='module config file (.pilotfwconfig.json)')
  parser.add_argument('name', metavar='node', type=str, nargs='?',
                      help='Specify named node to update firmware code (node is not reprogrammed, it is just used to get the modules)')
  parser.add_argument('--local', '-l', dest='local', action='store_true',
                  help='Use local instead of github template')
  for mod in range(1, PilotDriver.MODULE_COUNT+1):
    parser.add_argument('--module{}'.format(mod), '-m{}'.format(mod), dest='m{}'.format(mod), help='Firmware id of module {}'.format(mod))
      


import os
import json
import yaml
import hashlib
BLOCKSIZE = 65536

def get_node_from_config(args):
  #check if a .pilotfwconfig.json file exists and extract nodes
  config = get_config(args)
  if 'nodes' in config and isinstance(config['nodes'], list):

    specified_node = args.name if 'name' in args and args.name is not None else 'default'
    for node in config['nodes']:
      nodesfromconfig = True
      args.node = node['node']
      if 'user' in node:
        args.user = node['user']
      if 'user' in node:
        args.password = node['password']
      if 'hardware' in node:
        args.hardware = node['hardware']
      if specified_node == node['name']:
        print("Using node '{}'".format(specified_node))
        break
  
  return args

def get_modules_from_config(args):
  config = get_config(args)
  if 'modules' in config and isinstance(config['modules'], list):
    for module in config['modules']:
      if 'slot' in module and 'fid' in module:
        setattr(args, 'm{}'.format(module['slot']), module['fid'])
  return args

def update_firmware_version_in_config(args, version):
  config = get_config(args)
  config['firmware_version'] = version
  save_config(args, config)

def find_fw_toplevel(args):
  dir = args.workdir if args.workdir else os.getcwd() 
  olddir = ''
  while dir != olddir:
    olddir = dir
    if os.path.isfile(os.path.join(dir, '.pilotfwconfig.json')):
      return dir
    dir = os.path.abspath(os.path.join(dir, '..'))
  return ''

def get_compilers():
  compilers = []
  directories = []
  for subdir, dirs, files in os.walk(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'compiler')):
    for dir in dirs:
      if not dir.startswith("_"): #ignore dirs with underscore (to disable non-functioning compilers)
        compilerinfofile = os.path.join(subdir, dir, 'template.yml')
        if os.path.isfile(compilerinfofile):
          with open(compilerinfofile, 'r') as f:
            try:
              compilers.append(yaml.load(f.read(), Loader=yaml.FullLoader))
              directories.append(os.path.join(subdir, dir))
            except Exception as e: 
              print('Error parsing {}'.format(compilerinfofile))
              print(e)
  return compilers, directories

def show_compilers():
  compilers, _ = get_compilers()
  for compiler in compilers:
    print('{}: {} (extension: {})'.format(compiler['name'], compiler['description'], compiler['filter']))

def save_config(args, config):
    with open(os.path.join(args.workdir, '.pilotfwconfig.json') if args.workdir else './.pilotfwconfig.json', 'w') as configfile:
      json.dump(config, configfile)

def get_config(args):
  try:
    if args.configfile == None:
      configfile = os.path.join(args.workdir, '.pilotfwconfig.json') if args.workdir else './.pilotfwconfig.json'
      if os.path.isfile(configfile):
        args.configfile = configfile
      else:
        configfile = os.path.join(find_fw_toplevel(args), '.pilotfwconfig.json')
        if os.path.isfile(configfile):
          args.configfile = configfile
        else:
          args.configfile = os.environ['PILOT_CONFIG']
        
    if os.path.isfile(args.configfile):
      hasher = hashlib.sha1()
      with open(args.configfile) as configfile:
        config = json.load(configfile)

      with open(args.configfile, 'rb') as afile:
          buf = afile.read(BLOCKSIZE)
          while len(buf) > 0:
              hasher.update(buf)
              buf = afile.read(BLOCKSIZE)
      
      print('Using config file {}, SHA1={}'.format(args.configfile, hasher.hexdigest()))

      return config
    else:
      print("Configuration file (usually .pilotfwconfig.json) not found, exiting")
      exit(1)
  except:
    print("Error reading configuration file. Check if it is valid JSON")
    exit(1)

def get_model(args):
  model = {}
  #if no source path parameter is given try the environment var
  if args.source == None:
    try:
      source = os.path.join(args.workdir, 'basefw/stm') if args.workdir else './basefw/stm'
      if os.path.isdir(source):
        args.source = source
      else: 
        args.source = os.environ['PILOT_SOURCE']
    except:
      print("No parameter source directory given, no './basefw/stm' folder found and no PILOT_SOURCE environment variable defined, exiting")
      exit(1) 
  with open(os.path.join(os.path.abspath(args.source), 'stmmodel.json')) as stmmodelfile:
    model.update(json.load(stmmodelfile))
    model['devices'] = []
  return model


import math
import copy

io_to_port = [
  3, 1, 2, 0, 
  6, 5, 9, 8,
  11, 10, 14, 13,
  16, 15, 18, 19,
  4, 7, 12, 17
]

ports = [
    { "io": 0,  "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 3
    { "io": 1,  "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 1
    { "io": 2,  "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 2
    { "io": 3,  "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 0
    { "io": 4,  "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 6
    { "io": 5,  "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 5
    { "io": 6,  "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 9
    { "io": 7,  "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 8
    { "io": 8,  "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 11
    { "io": 9,  "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 10
    { "io": 10, "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 14
    { "io": 11, "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 13
    { "io": 12, "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 16
    { "io": 13, "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 15
    { "io": 14, "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 18
    { "io": 15, "direction": "in", "averages": 128, "range": "0-10V" }  # PORT 19
]

internalports = [
    { "io": 16, "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 4
    { "io": 17, "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 7
    { "io": 18, "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 12
    { "io": 19, "direction": "in", "averages": 128, "range": "0-10V" }, # PORT 17
]

def getDevice(model, module, compiler, helpers):
  return AIO20Device(model, module, compiler, helpers)

def toGPIO(this, items):
  return 'GPIO' + chr(items+65)

def default_config():
  return { 'ports': ports }
class AIO20Device():
  size = 40
  ctype = 'uint16_t[16]'
  rusttype = 'u8'
  include = ['aio20.h'] 
  init_source = ''
  dev_to_mem_source = ''
  mem_to_dev_source = ''
  mem_doc = {
    "read": [
    {
      "byte": 2,
      "bit": 0,
      "length": 16,
      "name": "aio0",
      "desc": "Analog I/O 0",
      "signed": False
    },
    {
      "byte": 0,
      "bit": 0,
      "length": 16,
      "name": "aio1",
      "desc": "Analog I/O 1",
      "signed": False
    },
    {
      "byte": 6,
      "bit": 0,
      "length": 16,
      "name": "aio2",
      "desc": "Analog I/O 2",
      "signed": False
    },
    {
      "byte": 4,
      "bit": 0,
      "length": 16,
      "name": "aio3",
      "desc": "Analog I/O 3",
      "signed": False
    },
    {
      "byte": 12,
      "bit": 0,
      "length": 16,
      "name": "aio4",
      "desc": "Analog I/O 4",
      "signed": False
    },
    {
      "byte": 10,
      "bit": 0,
      "length": 16,
      "name": "aio5",
      "desc": "Analog I/O 5",
      "signed": False
    },
    {
      "byte": 18,
      "bit": 0,
      "length": 16,
      "name": "aio6",
      "desc": "Analog I/O 6",
      "signed": False
    },
    {
      "byte": 16,
      "bit": 0,
      "length": 16,
      "name": "aio7",
      "desc": "Analog I/O 7",
      "signed": False
    },
    {
      "byte": 22,
      "bit": 0,
      "length": 16,
      "name": "aio8",
      "desc": "Analog I/O 8",
      "signed": False
    },
    {
      "byte": 20,
      "bit": 0,
      "length": 16,
      "name": "aio9",
      "desc": "Analog I/O 9",
      "signed": False
    },
    {
      "byte": 28,
      "bit": 0,
      "length": 16,
      "name": "aio10",
      "desc": "Analog I/O 10",
      "signed": False
    },
    {
      "byte": 26,
      "bit": 0,
      "length": 16,
      "name": "aio11",
      "desc": "Analog I/O 11",
      "signed": False
    },
    {
      "byte": 32,
      "bit": 0,
      "length": 16,
      "name": "aio12",
      "desc": "Analog I/O 12",
      "signed": False
    },
    {
      "byte": 30,
      "bit": 0,
      "length": 16,
      "name": "aio13",
      "desc": "Analog I/O 13",
      "signed": False
    },
    {
      "byte": 36,
      "bit": 0,
      "length": 16,
      "name": "aio14",
      "desc": "Analog I/O 14",
      "signed": False
    },
    {
      "byte": 38,
      "bit": 0,
      "length": 16,
      "name": "aio15",
      "desc": "Analog I/O 15",
      "signed": False
    },
    {
      "byte": 8,
      "bit": 0,
      "length": 16,
      "name": "r1",
      "desc": "Analog Frontend 1 Id Resistor",
      "signed": False
    },
    {
      "byte": 14,
      "bit": 0,
      "length": 16,
      "name": "r2",
      "desc": "Analog Frontend 2 Id Resistor",
      "signed": False
    },
    {
      "byte": 24,
      "bit": 0,
      "length": 16,
      "name": "r3",
      "desc": "Analog Frontend 3 Id Resistor",
      "signed": False
    },
    {
      "byte": 34,
      "bit": 0,
      "length": 16,
      "name": "r4",
      "desc": "Analog Frontend 4 Id Resistor",
      "signed": False
    }
  ],
  "write": []
  }

  decl = ''

  module = None
  model = None
  compiler = None
  helpers = None 
  
  def __init__(self, model, module, compiler, helpers):
    self.size = 40
    self.module = module
    self.helpers = {'gpio': toGPIO, **helpers}
    self.compiler = compiler
    self.model = model


  def compile(self):

    if "config" not in self.module:
      self.module['config'] = { 'ports': copy.deepcopy(ports) }

    if "custom" not in self.model:
      self.model["custom"] = { }
    
    if "aio20" not in self.model["custom"]:
      self.model["custom"]["aio20"] = []

    res, err = self.parse_port_config(self.module['config']['ports'])
    if err:
      print(err)
      exit(1)
    
    self.model["custom"]["aio20"].append({ 'index': self.module['slot']-1, 'config': res })

    dev_to_mem_str = """// source for device {{device.name}}
  get_module_info()->m{{device.index}}_status |= single_ended_adc_read_all({{device.index}}, (uint8_t *) &plc_mem_devices.m{{device.slot}}); 
  """

    init_str = "  // initialization for device {{device.name}}\n  int32_t status = 1;\n  if (aio20_get_device_id({{device.index}}) == 0x424)\n  {\n    status = 0;\n    AIO20_init({{device.index}});\n  }\n  return status;"
    init_template = self.compiler.compile(init_str)
    dev_to_mem_template = self.compiler.compile(dev_to_mem_str)

    self.dev_to_mem_source = dev_to_mem_template(self.module, self.helpers)
    self.init_source = init_template(self.module, self.helpers)

    c_decl_out = '\n'
    rust_decl_out = '\n'

    # DAC
    mem_to_dev_str = "  // source for device {{device.name}}\n"
    for out in res['port_out']:
      c_decl_out = c_decl_out + f"     uint16_t out{out['io']};\n"
      rust_decl_out = rust_decl_out + f"     pub out{out['io']}: u16, // {out['port']}\n"
      mem_to_dev_str = mem_to_dev_str + "  get_module_info()->m{{device.index}}_status |= single_ended_dac_write({{device.index}}, "+ str(out['io']) +", plc_mem_devices.m{{device.slot}}.out"+str(out['io'])+");\n"

    if len(res['port_out']) > 0:
      mem_to_dev_template = self.compiler.compile(mem_to_dev_str)
      self.mem_to_dev_source = mem_to_dev_template(self.module, self.helpers)

    # declarations
    self.decl = {
       'c': { 'name': 'aio20_' + str(self.module['slot']-1) + '_t', 'decl': """typedef struct { 
     uint16_t in3;
     uint16_t in1;
     uint16_t in2;
     uint16_t in0;
     uint16_t r1;
     uint16_t in5;
     uint16_t in4;
     uint16_t r2;
     uint16_t in7;
     uint16_t in6;
     uint16_t in9;
     uint16_t in8;
     uint16_t r3;
     uint16_t in11;
     uint16_t in10;
     uint16_t in13;
     uint16_t in12;
     uint16_t r4;
     uint16_t in14;
     uint16_t in15;""" + c_decl_out + """
   } aio20_""" + str(self.module['slot']-1) + """_t;
   """},
       'rust': { 'name': 'Aio20_'+ str(self.module['slot']-1), 'decl': """#[repr(C)]
   pub struct Aio20_""" + str(self.module['slot']-1) + """ { 
     pub in3: u16,  // 0
     pub in1: u16,  // 1
     pub in2: u16,  // 2
     pub in0: u16,  // 3
     r1: u16,        // 4
     pub in5: u16,  // 5
     pub in4: u16,  // 6
     r2: u16,        // 7
     pub in7: u16,  // 8
     pub in6: u16,  // 9
     pub in9: u16,  // 10
     pub in8: u16,  // 11
     r3: u16,        // 12
     pub in11: u16, // 13  
     pub in10: u16, // 14
     pub in13: u16, // 15
     pub in12: u16, // 16
     r4: u16,        // 17
     pub in14: u16, // 18 
     pub in15: u16, // 19""" + rust_decl_out + """
   }
   """ }
     }
  
  def parse_port_config(self, ports):
      port_cfgs = []
      port_dacs = []
      port_out = []
      for intport in internalports:
        ports.append(intport)
      
      # check if all items have io
      for port in ports:
        if not 'io' in port:
            return {}, "'io' missing in port" # Error format?
        port['port'] = io_to_port[port['io']]

      ports = sorted(ports, key=lambda item: item['port'])

      # check if ports are exhaustive
      ios = list(map(lambda p: p['port'], ports))
      all_ios = [i for i in range(0, 20)]
      if (len(ios) != len(all_ios) or ios != all_ios):
        return {}, f"IO list in configuration of '{self.module['fid_nicename']}' in slot {self.module['slot']} is not exhaustive. All 16 IOs need to be declared"

      for port in ports:
          # 1111xxxxxxxxxxxx PortCfgFuncID Port function / mode
          # xxxx1xxxxxxxxxxx funcprm_avrInv AVR / INV
          # xxxxx111xxxxxxxx funcprm_range DAC Range / ADC Range
          # xxxxxxxx111xxxxx funcprm_nsamples Number of samples / CAP
          # xxxxxxxxxxx11111 funcprm_port Associated port 0..31
  
          err, port_cfg, port_dac = parse_direction(port)
          if err != "":
              return {}, err

          if port_dac != 0: # dac configured
            port_out.append({'io': port['io'], 'port': port['port']})
      
          err, cfg = parse_averages(port)
          if err == "": # no error
              port_cfg = port_cfg | cfg
          else:
              return {}, err
      
          err, cfg = parse_range(port)
          if err == "": # no error
              port_cfg = port_cfg | cfg
          else:
              return {}, err
          port_cfgs.append(port_cfg)
          port_dacs.append(port_dac)
      
      return {
        "port_cfgs": ", ".join("0x{:02X}".format(p) for p in port_cfgs), 
        "port_dacs": ", ".join("0x{:02X}".format(p) for p in port_dacs),
        "port_out": port_out
        }, ""


config = [
    { "io": 0, "direction": "in", "averages": 128, "range": "0-10V" } 
]

def parse_averages(port):
    if 'averages' in port:
        if isinstance(port['averages'], int) and port['averages'] <= 128:
            return "", int(math.log2(port['averages'])) << 5
        else:
            return f"port {port['io']} has unknown value '{port['average']}' for 'average'", 0
    else: # we assume 128
        return "",  7 << 5

def parse_range(port):
    if 'range' in port:
        if port['range'].lower() == '0-10v':
            return "", 0b00100000000
        elif port['range'].lower() == '0-2.5v':
            return "", 0b10000000000
        else:
            return f"port {port['io']} has unknown value '{port['range']}' in 'range'", 0
    else: # we assume 0-10V
        return "", 0b00100000000


def parse_direction(port):
    if 'direction' in port:
        if port['direction'].lower() == 'in': # adc
            return "", 0x7000, 0
        elif port['direction'].lower() == 'outmon': # dac with adc monitoring
            return "", 0x6000, 0x0800
        elif port['direction'].lower() == 'out': # dac
            return "", 0x5000, 0x0666
        else:
            return f"port {port['io']} has unknown value '{port['direction']}' in 'direction'", 0, 0
    else: # we assume in  (adc)
        return "", 0x7000, 0


    
    


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from . import pilot_api_pb2 as pilot__api__pb2

class PilotApiStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetFwConfig = channel.unary_unary(
                '/pilot_api.PilotApi/GetFwConfig',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=pilot__api__pb2.FwConfigResponse.FromString,
                )
        self.GetID = channel.unary_unary(
                '/pilot_api.PilotApi/GetID',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=pilot__api__pb2.IdResponse.FromString,
                )
        self.SetName = channel.unary_unary(
                '/pilot_api.PilotApi/SetName',
                request_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_wrappers__pb2.BoolValue.FromString,
                )
        self.SetDebugMode = channel.unary_unary(
                '/pilot_api.PilotApi/SetDebugMode',
                request_serializer=pilot__api__pb2.DebugModeMsg.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_wrappers__pb2.BoolValue.FromString,
                )
        self.GetDebugMode = channel.unary_unary(
                '/pilot_api.PilotApi/GetDebugMode',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=pilot__api__pb2.DebugModeMsg.FromString,
                )
        self.GetMetricInfo = channel.unary_stream(
                '/pilot_api.PilotApi/GetMetricInfo',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=pilot__api__pb2.MetricInfo.FromString,
                )
        self.GetFloatMetric = channel.unary_stream(
                '/pilot_api.PilotApi/GetFloatMetric',
                request_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_wrappers__pb2.FloatValue.FromString,
                )
        self.GetMetric = channel.unary_stream(
                '/pilot_api.PilotApi/GetMetric',
                request_serializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
                response_deserializer=pilot__api__pb2.MetricMsg.FromString,
                )
        self.SetFirmware = channel.unary_stream(
                '/pilot_api.PilotApi/SetFirmware',
                request_serializer=pilot__api__pb2.FirmwareRequest.SerializeToString,
                response_deserializer=pilot__api__pb2.FirmwareStatus.FromString,
                )


class PilotApiServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetFwConfig(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetID(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetName(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetDebugMode(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDebugMode(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetMetricInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFloatMetric(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetMetric(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetFirmware(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PilotApiServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetFwConfig': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFwConfig,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=pilot__api__pb2.FwConfigResponse.SerializeToString,
            ),
            'GetID': grpc.unary_unary_rpc_method_handler(
                    servicer.GetID,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=pilot__api__pb2.IdResponse.SerializeToString,
            ),
            'SetName': grpc.unary_unary_rpc_method_handler(
                    servicer.SetName,
                    request_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
                    response_serializer=google_dot_protobuf_dot_wrappers__pb2.BoolValue.SerializeToString,
            ),
            'SetDebugMode': grpc.unary_unary_rpc_method_handler(
                    servicer.SetDebugMode,
                    request_deserializer=pilot__api__pb2.DebugModeMsg.FromString,
                    response_serializer=google_dot_protobuf_dot_wrappers__pb2.BoolValue.SerializeToString,
            ),
            'GetDebugMode': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDebugMode,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=pilot__api__pb2.DebugModeMsg.SerializeToString,
            ),
            'GetMetricInfo': grpc.unary_stream_rpc_method_handler(
                    servicer.GetMetricInfo,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=pilot__api__pb2.MetricInfo.SerializeToString,
            ),
            'GetFloatMetric': grpc.unary_stream_rpc_method_handler(
                    servicer.GetFloatMetric,
                    request_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
                    response_serializer=google_dot_protobuf_dot_wrappers__pb2.FloatValue.SerializeToString,
            ),
            'GetMetric': grpc.unary_stream_rpc_method_handler(
                    servicer.GetMetric,
                    request_deserializer=google_dot_protobuf_dot_wrappers__pb2.StringValue.FromString,
                    response_serializer=pilot__api__pb2.MetricMsg.SerializeToString,
            ),
            'SetFirmware': grpc.unary_stream_rpc_method_handler(
                    servicer.SetFirmware,
                    request_deserializer=pilot__api__pb2.FirmwareRequest.FromString,
                    response_serializer=pilot__api__pb2.FirmwareStatus.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'pilot_api.PilotApi', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class PilotApi(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetFwConfig(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/pilot_api.PilotApi/GetFwConfig',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            pilot__api__pb2.FwConfigResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/pilot_api.PilotApi/GetID',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            pilot__api__pb2.IdResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/pilot_api.PilotApi/SetName',
            google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
            google_dot_protobuf_dot_wrappers__pb2.BoolValue.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetDebugMode(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/pilot_api.PilotApi/SetDebugMode',
            pilot__api__pb2.DebugModeMsg.SerializeToString,
            google_dot_protobuf_dot_wrappers__pb2.BoolValue.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDebugMode(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/pilot_api.PilotApi/GetDebugMode',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            pilot__api__pb2.DebugModeMsg.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetMetricInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/pilot_api.PilotApi/GetMetricInfo',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            pilot__api__pb2.MetricInfo.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFloatMetric(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/pilot_api.PilotApi/GetFloatMetric',
            google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
            google_dot_protobuf_dot_wrappers__pb2.FloatValue.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetMetric(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/pilot_api.PilotApi/GetMetric',
            google_dot_protobuf_dot_wrappers__pb2.StringValue.SerializeToString,
            pilot__api__pb2.MetricMsg.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetFirmware(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/pilot_api.PilotApi/SetFirmware',
            pilot__api__pb2.FirmwareRequest.SerializeToString,
            pilot__api__pb2.FirmwareStatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



#!/usr/bin/env python3
import os
import sys
import argparse
import subprocess
import json
from distutils.dir_util import copy_tree
from shutil import copyfile, move
import yaml
import importlib.util
from .plc import Plc
from . import helper
import lazy_import
from colorama import Fore
Compiler = lazy_import.lazy_callable("pybars.Compiler")

def templateparser(args, dir, model, compiler, helpers):
  for subdir, dirs, files in os.walk(dir):
    outdir = os.path.join(args.workdir, os.path.relpath(subdir, dir))
    for file in files:
      infile = os.path.join(subdir, file)
      outfilename = os.path.basename(infile)
      with open(infile) as f:
        if infile.endswith('.templ'):
          template = compiler.compile(f.read())
          output = template(model, helpers)
          outfilename = outfilename[:-6] #.replace('.templ', '')
        else:
          output = f.read()
        with open(os.path.join(outdir, outfilename), 'w+') as f:
          f.write(output)  

def pluginparser(args, model, compiler, helpers):
  plugindir = os.path.join(args.workdir, 'plugins', 'template')
  if os.path.isdir(plugindir):
    #dirs = os.listdir(plugindir)
    #for dir in dirs:
    #  templateparser(args, os.path.join(plugindir, dir),  model, compiler, helpers )
    templateparser(args, plugindir,  model, compiler, helpers )

def _hex(this, items):
    return hex(items)

def parseplcvariables(varcsvfilepath ):
  with open(varcsvfilepath) as varcsvfile:
    content = [line.strip() for line in varcsvfile]

  mode = 'variables'
  variables = []
  for line in content:
    if line.startswith('//'): 
      mode = line[2:].strip().lower()
    else:
      columns = line.split(';')
      if mode == 'variables' and len(columns) >= 5:
        varname = columns[2].split('.')
        if len(varname) > 2 and varname[0].upper() == "CONFIG":
          var = {'number': int(columns[0]), 'location': columns[1], 'resource': varname[1], 'type': columns[4]}
          if len(varname) == 3:
            var['instance'] = None
            var['name'] = varname[2]
          elif len(varname) >= 4:
            var['instance'] = varname[2]
            var['name'] = '.'.join(varname[3:])
          variables.append(var)
  return variables

def createdoc(args, model, config):
  docfile = os.path.join(args.workdir, 'out/fwconfig.json') if args.workdir else './out/fwconfig.json'
  doc = { "modules": [], "variables": [] }

  # create module documentation
  for m in model['memmodules']:
    mod_cfg = next(iter([x for x in config['modules'] if x['slot'] == m['slot']]))
    n = mod_cfg['name'] if 'name' in mod_cfg else mod_cfg['device']['name']
    mem_doc = m['device']['spec'].mem_doc
    
    # add custom label
    if 'config' in mod_cfg and 'labels' in mod_cfg:
      for label in mod_cfg['labels']:
        was_found = False
        for dir in ['read', 'write']:
          found = next((x for x in mem_doc[dir] if x['name'] == label), None)
          if found:
            found['label'] = mod_cfg['labels'][label]
            was_found = True
        if not was_found:
          print(Fore.YELLOW + "Warning: Custom label '{}' defined in module {} could not be matched to a module property".format(label, m['slot']))
          print(Fore.YELLOW + "Possible label values are: " + ', '.join(list(map(lambda x: x['name'], mem_doc['read'])) + list(map(lambda x: x['name'], mem_doc['write']))))
  
    doc['modules'].append({
      "slot": m['slot'],
      "fid": m['fid'],
      "fid_nicename": m['fid_nicename'],
      "name": n,
      "doc": mem_doc
    })

  # create variable documentation
  varfile = os.path.join(args.workdir, 'out/VARIABLES.csv') if args.workdir else './out/VARIABLES.csv'
  doc['variables'] = parseplcvariables(varfile)

  with open((docfile), 'w') as f:
    json.dump(doc, f)

def main(args, version):
  parser = Compiler()
  helpers = {'hex': _hex}
  compilerdirectory = None
  if ('show_compilers' in args and args.show_compilers):
    helper.show_compilers()
  elif (args.fw_subparser_name == 'build'):

    # load config file
    config = helper.get_config(args)
    # load model file
    model = helper.get_model(args) 
    # get toplevel dir    
    projectdir = helper.find_fw_toplevel(args)

    if 'config' in config:
      model['config'] = config['config']

    for module in model['Modules']:
      configmodule = next( (x for x in config['modules'] if int(x['slot']) == module['Slot'] + 1 ), None)
      if configmodule:
        if configmodule['fid'] != module['Name']:
          print("Base firmware is not matching configuration on module {}. Basefirmware has '{}' but config has '{}'. Please update base firmware or change config.".format(module['Slot'] + 1, module['Name'], configmodule['fid']))
          exit(1)
      elif module['Name'] != 'None':
        print("WARNING: Base firmware has '{}' in module {}, but no configuration.".format(module['Name'], module['Slot'] + 1))

    compilers, directories = helper.get_compilers()
    for index, compiler in enumerate(compilers):
      if compiler['name'] == config['compiler']:
        compilerdirectory = directories[index]
    if compilerdirectory == None:
      print('Compiler {} defined in configuration file not found.'.format(config['compiler']))
      return 1

    # generate common C code for module I/O 
    plc = Plc(args, version, model, config, parser, helpers)
    templateparser(args, os.path.join(os.path.abspath(os.path.dirname(__file__)), 'compiler', 'template'), plc.model, parser, helpers)

    pluginparser(args, plc.model, parser, helpers)
    # language specific implementation for processing module I/O and custom logic
    spec = importlib.util.spec_from_file_location("module.name", os.path.join(compilerdirectory, 'compiler.py'))
    compiler = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(compiler)
    buildresult = compiler.main(args, config, model, projectdir, compilerdirectory)

    if buildresult == 0:
        #create documentation json
        createdoc(args, model, config)
     
  return buildresult      



import os
import json
import argparse

from pilot.grpc_gen.pilotbuild_pb2 import BinaryType
from . import arguments
from . import helper 
from .sbc import Sbc
from .pilotserver import PilotServer
from .pilotdriver import PilotDriver


def main(args):
  args = helper.get_node_from_config(args)
  program(args)

def program(args):
  try:
    with Sbc(args) as sbc:
      # PilotServer
      pilotserver = PilotServer(sbc)
      if args.server != None:
        pilotserver.pilot_server = args.server
      
      #PilotDriver
      pilotdriver = PilotDriver(pilotserver, sbc)

      if not pilotdriver.driver_loaded():
        print('Drivers are not loaded. Please use --node to specify node IP if you connect remotely or install pilot drivers first by running sudo pilot setup.')
        return 2

      #if not pilotdriver.check_raspberry() and not args.node:
      #  print('This does not seem to be a Raspberry Pi. Please use the --node option to remote connect to it.')
      #  return 2

      if args.vars:
        if not os.path.isfile(args.vars):
          print('You need to specify a valid file for the --variables attribute.')
          exit(1)
      else:
        args.vars = os.path.join(args.workdir, 'out/VARIABLES.csv') if args.workdir else './out/VARIABLES.csv'
        if os.path.isfile(args.vars):
          print('Using variable file ' + args.vars)

      if args.doc:
        if not os.path.isfile(args.doc):
          print('You need to specify a valid doc (fwconfig.json) file for the --doc attribute.')
          exit(1)
      else:
        args.doc = os.path.join(args.workdir, 'out/fwconfig.json') if args.workdir else './out/fwconfig.json'
        if os.path.isfile(args.doc):
          print('Using documentation file ' + args.doc)

      if args.bin:
        if not os.path.isfile(args.bin):
          print('You need to specify a valid file for the --binary attribute.')
          exit(1)
      elif os.path.isfile(os.path.join(args.workdir, 'out/stm.bin') if args.workdir else './out/stm.bin'):
        args.bin = os.path.join(args.workdir, 'out/stm.bin') if args.workdir else './out/stm.bin'
      else:
        print('You need to specify an image file to write with the --binary attribute.')
        exit(1)

      if args.logicbin:
        if not os.path.isfile(args.logicbin):
          print('You need to specify a valid CPLD file for the --logicbinary attribute.')
          exit(1)
      elif os.path.isfile(os.path.join(args.workdir, 'basefw/cpld/output_files/cpld.jam') if args.workdir else './basefw/cpld/output_files/cpld.jam'):
        args.logicbin = os.path.join(args.workdir, 'basefw/cpld/output_files/cpld.jam') if args.workdir else './basefw/cpld/output_files/cpld.jam'
      else:
        print('You need to specify an image file to write with the --logicbinary attribute.')
        exit(1)

      files = {
          BinaryType.MCUFirmware: args.bin,
          BinaryType.FPGABitstream: args.logicbin,
          BinaryType.Variables: args.vars,
          BinaryType.Docs: args.doc
      }
      pilotdriver.program(files, program_cpld=not (args.mcuonly==True), program_mcu=True, bootmsg=args.wait_bootmsg, reload_driver=not (args.mcuonly==True)) #only reload driver if cpld is programmed (indicates hardware change)
  except Exception as error:
    print(error)

if (__name__ == "__main__"):
  parser = argparse.ArgumentParser(
    description='Write custom firmware')
  arguments.program_arguments(parser)
  main(parser.parse_args())


from __future__ import absolute_import, division, print_function, with_statement
from __future__ import unicode_literals

from deepstreampy import connect
from deepstreampy import rpc

from tornado import gen
from tornado import ioloop

import signal


def sq(x, response):
    """
    An RPC handler function that sends the square root of the input.
    """
    if isinstance(x, int):
        response.send(x * x)
    else:
        response.error("sq received unexpected input")


@gen.coroutine
def run():
    # First we set up the provider
    provider = yield connect("ws://localnode:6020/deepstream")
    yield provider.login({"username": "A"})
    provider.on('error', lambda msg, event, topic: print('a provider error occured'))

    yield provider.rpc.provide("sq", sq)

    # Then we set up the consumer
    consumer = yield connect("ws://localnode:6020/deepstream")
    yield consumer.login({"username": "B"})
    consumer.on('error', lambda msg, event, topic: print('a consumer error occured'))

    # Next we make an RPC call to the "sq" function
    result = yield consumer.rpc.make("sq", 2)
    print(result)
    # Output: 4

    # In case the provider ends up sending an error message, an exception
    # with that message will be raised when trying to get the result of the
    # call.
    try:
        yield consumer.rpc.make("sq", "2")
    except rpc.RPCException as e:
        print(e)
        # Output: sq received unexpected input


if __name__ == "__main__":
    run()
    signal.signal(signal.SIGINT, lambda _, __: ioloop.IOLoop.current().stop())
ioloop.IOLoop.current().start()



#!/usr/bin/env python3

from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
import os
import tarfile
from pathlib import Path

here = os.path.abspath(os.path.dirname(__file__))

# Get the version
with open(os.path.join(here, 'pilot', 'VERSION'), encoding='utf-8') as f:
  version = f.read().strip()

# Get the long description from the README file
with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
  long_description = f.read()

# extra files
def package_files(directoryarr):
    paths = []
    for directory in directoryarr:
        if directory == 'project': #additionally pack these
            project_dir = os.path.join('pilot', directory)
            compiler_dirs = os.listdir(project_dir)
            for compiler_dir in compiler_dirs:
                project_templ_path = os.path.join(project_dir, compiler_dir)
                project_templates = os.listdir(project_templ_path)
                for proj_templ in project_templates:
                    ptf = os.path.join(project_templ_path, proj_templ)
                    if Path(ptf).is_dir():
                        tar_file = ptf+'.tar.gz'
                        tar_dir(tar_file, ptf)
                        paths.append(os.path.join('..', tar_file))
                print(project_templates)
            pass
        else:
            for (path, _directories, filenames) in os.walk(os.path.join('pilot', directory)):
                for filename in filenames:
                    if not filename.endswith(".pyc"):
                        paths.append(os.path.join('..', path, filename))
    return paths

def tar_dir(name, directory):
    with tarfile.open(name, "w:gz") as tar:
        for (path, _directories, filenames) in os.walk(directory):
            for filename in filenames:
                full_file = os.path.join('.', path, filename)
                arcname=os.path.relpath(full_file, directory)
                tar.add(full_file, arcname=arcname)
        tar.close()

extra_files = package_files(['bin', 'compiler', 'plugins', 'devices', 'project'])
extra_files.append('VERSION')
extra_files.append('configdefs.json')
extra_files.append('targethardware.json')

setup(
  name='pilot-config',
  version=version,
  description='Pilot Automation Command Line Utility',
  long_description=long_description,
  author='Daniel Amesberger',
  author_email='daniel.amesberger@amescon.com',
  url='https://www.amescon.com',
  classifiers=[
      # How mature is this project? Common values are
      #   3 - Alpha
      #   4 - Beta
      #   5 - Production/Stable
      'Development Status :: 3 - Alpha',
      'Intended Audience :: Developers',
      'Topic :: Software Development :: Build Tools',
      'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
      'Programming Language :: Python :: 3',
  ],
  keywords='pilot development automation plc',
  packages=find_packages(exclude=[]),
  package_data={'': extra_files},
  install_requires=['lazy_import',
                    'pyYAML',
                    'pybars3',
                    'halo',
                    'requests',
                    'argparse',
                    'bugsnag',
                    'uuid',
                    'bugsnag',
                    'colorama',
                    'paramiko',
                    'scp',
                    'pyjwt',
                    'qrcode_terminal',
                    'gql',
                    'rich',
                    'protobuf',
                    'grpcio',
                    'requests_toolbelt'
                    ],
  python_requires='>=3',
  entry_points={  # Optional
      'console_scripts': [
          'pilot=pilot.pilot:main',
      ],
  },
  project_urls={  # Optional
    'Bug Reports': 'https://github.com/pypa/sampleproject/issues',
    'Funding': 'https://donate.pypi.org',
    'Say Thanks!': 'http://saythanks.io/to/example',
    'Source': 'https://github.com/pypa/sampleproject/',
  },
)


import argparse

# class imports
from .pilotdriver import PilotDriver
from .pilotserver import PilotServer
from .sbc import Sbc

from colorama import Fore
from colorama import Style
from colorama import init


def arguments(parser):
  # parser.add_argument('--terminal', '-t', action='store_true', help='forces the terminal version instead of GUI')
  parser.add_argument('--help', '-h', default=None, dest='source',
                      help='Get help on modules')

def main(args):
  try:
    with Sbc(args) as sbc:
      pilotserver = PilotServer(sbc)
      if 'server' in args and args.server != None:
        pilotserver.pilot_server = args.server
      
      #PilotDriver
      pilotdriver = PilotDriver(pilotserver, sbc)

      if not pilotdriver.driver_loaded():
        print('Drivers are not loaded. Please use --node if you connect remotely or install pilot drivers first by running sudo pilot setup.')
        return 2

      #if not pilotdriver.check_raspberry() and not args.node:
      #  print('This does not seem to be a Raspberry Pi. Please use the --node option to remote connect to it.')
      #  return 2

      if sbc.need_sudo_pw():
        print('we need sudo on remote Node (without interactive authentication)')
        return 2

      modules, success = pilotdriver.load_pilot_defs()
      
      if not success:
        print('Could not get module data.')
        return 1

      #Print module help
      moduleinfos = pilotdriver.get_help()

      if moduleinfos:
        for module in moduleinfos:
          print()
          print(Fore.GREEN + 'Module [{}] {}:'.format(module['number'], modules[module['number']-1]['currentfid_nicename']))
          print(module['help'])
          if module['examples'] and len(module['examples']) > 0:
            for idx,example in enumerate(module['examples']):
              print(Fore.CYAN + 'Example {}: {}'.format(idx+1, example['title']))
              print(example['example'])
  except Exception as error:
    print(error)
    exit(1)
  return 0
        
if __name__ == '__main__':
  parser = argparse.ArgumentParser(description='Module Info')
  arguments(parser)
  main(parser.parse_args())


#!/usr/bin/env python3

from __future__ import print_function  # disables annoying pylint print warning

import lazy_import

from pilot.grpc_gen.pilotbuild_pb2 import BinaryType
from . import arguments

sys = lazy_import.lazy_module("sys")
json = lazy_import.lazy_module("json")
re = lazy_import.lazy_module("re")
shlex = lazy_import.lazy_module("shlex")
time = lazy_import.lazy_module("time")
os = lazy_import.lazy_module("os")
argparse = lazy_import.lazy_module("argparse")
base64 = lazy_import.lazy_module("base64")
gettext = lazy_import.lazy_module("gettext")
bugsnag = lazy_import.lazy_module("bugsnag")
logging = lazy_import.lazy_module("logging")
paramiko = lazy_import.lazy_module("paramiko")

from uuid import getnode as get_mac
from bugsnag.handlers import BugsnagHandler
from threading import Thread
from colorama import Fore
from colorama import Style
from colorama import init

# class imports
from .pilotdriver import PilotDriver
from .pilotserver import PilotServer
from .sbc import Sbc

############### INIT ###################

init(autoreset=True)  # colorama color autoreset

DEBUG = False
EXECVP_ENABLED = False

############## PROC FILE ACCESS #####################


def main(args):

    logger = logging.getLogger()
    handler = BugsnagHandler()
    # send only ERROR-level logs and above
    handler.setLevel(logging.ERROR)
    logger.addHandler(handler)

    logging.getLogger("paramiko").setLevel(logging.ERROR)

    trywritedefaultfirmware = False
    result = 0
    try:
        with Sbc(args) as sbc:
            # PilotServer
            pilotserver = PilotServer(sbc)
            if args.server != None:
                pilotserver.pilot_server = args.server

            #PilotDriver
            pilotdriver = PilotDriver(pilotserver, sbc)

            #if not pilotdriver.check_raspberry() and not args.node:
            #  print('This does not seem to be a Raspberry Pi. Please use the --node option to remote connect to it.')
            #  return 2

            if not args.node and os.getuid() != 0:
                print('Please run with sudo permissions.')
                return 2

            if not args.regnode:
                if sbc.need_sudo_pw():
                    print(
                        'we need sudo on remote machine (without interactive authentication)'
                    )
                    return 2

                ret = pilotdriver.check_driver()
                if ret != 0:
                    if ret == 1:
                        if args.node:
                            print('Reboot of remote Node required')
                        else:
                            print('Reboot required')
                        ch = input("Do you want to reboot now? [y/n]: ")
                        if (ch == 'y' or ch == 'yes'):
                            sbc.reboot()
                        return 0
                    return 1

                # do not continue if driveronly is specified
                if args.driveronly:
                    return 0

                modules, success = pilotdriver.load_pilot_defs()
                if not success:
                    print(
                        Fore.YELLOW + 
                        'Could not read module data. Maybe the firmware is outdated, trying to write base firmware image.'
                    )
                    trywritedefaultfirmware = True
                if modules != None:
                    if args.source == None:
                        while (True):
                            print()
                            modules_with_multiple_fids = []
                            for module in modules:
                                multiple_fids = len(module['fids']) > 1
                                if multiple_fids:
                                    modules_with_multiple_fids.append(
                                        int(module['module']))
                                if not trywritedefaultfirmware:
                                    print('Module {}: {}{} {}'.format(
                                        module['module'], Fore.GREEN,
                                        module['currentfid_nicename'],
                                        '*' if multiple_fids else ''))

                            ch = ''
                            modsel = '/'.join(
                                [str(x) for x in modules_with_multiple_fids])
                            if len(modules_with_multiple_fids) > 0:
                                print(
                                    'Modules marked with an Asterisk (*) have multiple firmware configurations'
                                )
                                print(
                                    'Press Module Number [{}] to change selected firmware.'
                                    .format(modsel))
                            if (args.noninteractive
                                    or trywritedefaultfirmware):
                                ch = 'y'
                            else:
                                ch = input(
                                    'Do you want to build and program the Pilot Nexus Firmware? [y/n{}]: '
                                    .format('/' + modsel if
                                            len(modules_with_multiple_fids) > 0
                                            else '')).strip().lower()
                            if ch == 'y' or ch == 'yes':
                                version, files = pilotdriver.build(args.fwversion)
                                print('Firmware version: {}'.format(version))
                                if BinaryType.MCUFirmware in files:
                                    result = pilotdriver.program(
                                        files,
                                        bootmsg=args.wait_bootmsg)
                                else:
                                    print('Could not write firmware, no MCU firmware found')

                                break
                            elif ch.isdigit() and int(ch) in range(
                                    1, pilotdriver.MODULE_COUNT +
                                    1) and len(modules_with_multiple_fids) > 0:
                                changemodulenr = int(ch)
                                print()
                                print('Select Firmware for Module {}:'.format(
                                    changemodulenr))
                                for idx, fid in enumerate(
                                        modules[changemodulenr - 1]['fids']):
                                    print('{}. {}{}'.format(
                                        idx + 1, Fore.GREEN, fid['name']))
                                ch = input('0=Cancel, [1-{}]: '.format(
                                    len(modules[changemodulenr - 1]['fids'])))
                                if (ch.isdigit() and int(ch) > 0 and
                                        int(ch) <= len(modules[changemodulenr -
                                                               1]['fids'])):
                                    pilotdriver.set_module_fid(
                                        changemodulenr,
                                        modules[changemodulenr -
                                                1]['fids'][int(ch) - 1]['fid'])
                                    modules, success = pilotdriver.load_pilot_defs(
                                    )
                            else:
                                break

                    else:
                        version, success = pilotdriver.get_firmware_source(os.path.abspath(args.source), args.fwversion)

                else:
                    print('No modules found, is the driver loaded?')
                fwconfig = {}
                fwconfig['modules'] = modules

                #check if node is registered
                #if not trywritedefaultfirmware:
                #  if pilotserver.decoded == None:
                #    ch = input("You not logged in to access the Pilot Nexus Cloud, do you want to authenticate? [y/n]: ")
                #    if (ch == 'y' or ch == 'yes'):
                #      pilotserver.authenticate()

                #  if pilotserver.decoded != None:
                #    node = pilotserver.getnode()
                #    if node != None:
                #      print("Your node is registered as '{}'".format(node['name']))
                #      pilotserver.updatenode(fwconfig)

                #    elif not args.noninteractive and not trywritedefaultfirmware:
                #      ch = input("Node is not registered, do you want to register it with the Pilot Cloud? [y/n]: ")
                #      if (ch == 'y' or ch == 'yes'):
                #        pilotserver.registernode(fwconfig)

            #elif not args.noninteractive: # --regnode param
            #  pilotserver.registernode(None)
    except Exception as error:
        print(error)
        exit(1)

    if result == 0:
        if trywritedefaultfirmware:
            print(
                "Default firmware written. Run the setup tool again to program firmware for your modules"
            )
        #else:
        #  print ("To get help on how to use the modules, run 'pilot --module'")
    return result


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Setup Pilot Nexus')
    arguments.setup_arguments(parser)
    sys.exit(main(parser.parse_args()))
